# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: pynumaflow/proto/reducer/reduce.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'pynumaflow/proto/reducer/reduce.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%pynumaflow/proto/reducer/reduce.proto\x12\treduce.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x98\x04\n\rReduceRequest\x12\x31\n\x07payload\x18\x01 \x01(\x0b\x32 .reduce.v1.ReduceRequest.Payload\x12;\n\toperation\x18\x02 \x01(\x0b\x32(.reduce.v1.ReduceRequest.WindowOperation\x1a\x9e\x01\n\x0fWindowOperation\x12=\n\x05\x65vent\x18\x01 \x01(\x0e\x32..reduce.v1.ReduceRequest.WindowOperation.Event\x12\"\n\x07windows\x18\x02 \x03(\x0b\x32\x11.reduce.v1.Window\"(\n\x05\x45vent\x12\x08\n\x04OPEN\x10\x00\x12\t\n\x05\x43LOSE\x10\x01\x12\n\n\x06\x41PPEND\x10\x04\x1a\xf5\x01\n\x07Payload\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12.\n\nevent_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\twatermark\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12>\n\x07headers\x18\x05 \x03(\x0b\x32-.reduce.v1.ReduceRequest.Payload.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"j\n\x06Window\x12)\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04slot\x18\x03 \x01(\t\"\xa7\x01\n\x0eReduceResponse\x12\x30\n\x06result\x18\x01 \x01(\x0b\x32 .reduce.v1.ReduceResponse.Result\x12!\n\x06window\x18\x02 \x01(\x0b\x32\x11.reduce.v1.Window\x12\x0b\n\x03\x45OF\x18\x03 \x01(\x08\x1a\x33\n\x06Result\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x0c\n\x04tags\x18\x03 \x03(\t\"\x1e\n\rReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\x32\x8a\x01\n\x06Reduce\x12\x43\n\x08ReduceFn\x12\x18.reduce.v1.ReduceRequest\x1a\x19.reduce.v1.ReduceResponse(\x01\x30\x01\x12;\n\x07IsReady\x12\x16.google.protobuf.Empty\x1a\x18.reduce.v1.ReadyResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pynumaflow.proto.reducer.reduce_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_REDUCEREQUEST_PAYLOAD_HEADERSENTRY']._loaded_options = None
  _globals['_REDUCEREQUEST_PAYLOAD_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_REDUCEREQUEST']._serialized_start=115
  _globals['_REDUCEREQUEST']._serialized_end=651
  _globals['_REDUCEREQUEST_WINDOWOPERATION']._serialized_start=245
  _globals['_REDUCEREQUEST_WINDOWOPERATION']._serialized_end=403
  _globals['_REDUCEREQUEST_WINDOWOPERATION_EVENT']._serialized_start=363
  _globals['_REDUCEREQUEST_WINDOWOPERATION_EVENT']._serialized_end=403
  _globals['_REDUCEREQUEST_PAYLOAD']._serialized_start=406
  _globals['_REDUCEREQUEST_PAYLOAD']._serialized_end=651
  _globals['_REDUCEREQUEST_PAYLOAD_HEADERSENTRY']._serialized_start=605
  _globals['_REDUCEREQUEST_PAYLOAD_HEADERSENTRY']._serialized_end=651
  _globals['_WINDOW']._serialized_start=653
  _globals['_WINDOW']._serialized_end=759
  _globals['_REDUCERESPONSE']._serialized_start=762
  _globals['_REDUCERESPONSE']._serialized_end=929
  _globals['_REDUCERESPONSE_RESULT']._serialized_start=878
  _globals['_REDUCERESPONSE_RESULT']._serialized_end=929
  _globals['_READYRESPONSE']._serialized_start=931
  _globals['_READYRESPONSE']._serialized_end=961
  _globals['_REDUCE']._serialized_start=964
  _globals['_REDUCE']._serialized_end=1102
# @@protoc_insertion_point(module_scope)
